@@ -1,4 +1,4 @@
-@adt public Pattern begin
+@adt Pattern begin
     Comment(::String)
     
     Root
@@ -18,6 +18,8 @@
     end
 end
 
+@export_use Pattern: *
+
 struct IgnoreFile
     path::String
     stmts::Vector{Pattern}
