@@ -10,6 +10,8 @@ include("traits.jl")
 include("types.jl")
 include("emit.jl")
 include("match.jl")
+include("use.jl")
+include("derive.jl")
 include("print.jl")
 
 end # ADT
