@@ -2,8 +2,6 @@ mutable struct VariantFieldTypes
     expr::Vector{Any}
     mask::Vector{Int}
     guess::Vector{Any}
-    variant_types::Vector{Vector{Any}}
-
     VariantFieldTypes() = new()
 end
 
@@ -36,67 +34,35 @@ function assign_type!(info::EmitInfo, def::ADTTypeDef)
     return info
 end
 
+# NOTE: we don't support using variant in the type field
+# because they cannot be checked efficiently/statically
+# one should check them manually at runtime
 function guess_type!(info::EmitInfo, def::ADTTypeDef)
-    function change_type(expr)
+    function check_type(expr)
         @switch expr begin
             @case ::Symbol
-                expr in info.variant_names && return def.name
+                type = guess_type(def.m, expr)
+                type isa Type && return type
+                expr in info.variant_names && error("cannot use variant in type field")
                 return expr
-            @case :(Union{$(types...)}) || :(&Union{$(types...)})
-                types = map(types) do type
-                    type in info.variant_names && return def.name
-                    return type
-                end
-                return Expr(:curly, :Union, types...)
-            @case ::Expr
-                return Expr(expr.head, map(change_type, expr.args)...)
+            @case Expr(:curly, name, types...)
+                map(check_type, types)
+                return guess_type(def.m, expr)
             @case _
-                return expr
+                return guess_type(def.m, expr)
         end
     end
 
     for variant in def.variants
         typeinfo = get!(VariantFieldTypes, info.typeinfo, variant)
         typeinfo.guess = map(variant.fieldtypes) do type
-            guess_type(def.m, change_type(type))
+            check_type(type)
         end
         typeinfo.expr = variant.fieldtypes
     end
     return info
 end
 
-function scan_variant_types!(info::EmitInfo, def::ADTTypeDef)
-    for variant in def.variants
-        typeinfo = get!(VariantFieldTypes, info.typeinfo, variant)
-        typeinfo.variant_types = Vector{Vector{Any}}(undef, length(variant.fieldtypes))
-        for (idx, expr) in enumerate(variant.fieldtypes)
-            list = scan_field_variant_types!([], expr, info.variant_names)
-            typeinfo.variant_types[idx] = list
-        end
-    end
-    return info
-end
-
-# NOTE: we ignore Foo{VariantType} since we cannot actually check it
-function scan_field_variant_types!(list::Vector{Any}, expr, variant_names::Vector{Symbol})
-    @switch expr begin
-        @case ::Symbol
-            expr in variant_names && push!(list, expr)
-        @case :(Union{$(types...)}) || :(&Union{$(types...)})
-            for type in types
-                scan_field_variant_types!(list, type, variant_names)
-            end
-        # @case :($name{$(types...)}) # Foo{VariantType}
-        #     contains_variant_type(types, variant_names) && push!(list, expr)
-        # Foo{VariantType, A} where VariantType, just happen to have the same name
-        @case Expr(:where, type, params)
-            type = Expronicon.mark_typevars(type, name_only.(params))
-            scan_field_variant_types!(list, type, variant_names)
-        @case _
-    end
-    return list
-end
-
 function scan_fields!(info::EmitInfo, def::ADTTypeDef)
     typeset = Dict{Any, Int}()
 
@@ -178,12 +144,11 @@ function EmitInfo(def::ADTTypeDef)
     assign_type!(info, def)
     guess_type!(info, def)
     scan_fields!(info, def)
-    scan_variant_types!(info, def)
     return info
 end
 
 """
-    @adt <public> name begin
+    @adt name begin
         variant1
         variant2(field1, field2)
 
@@ -197,7 +162,6 @@ Create an algebra data type (ADT).
 
 ### Arguments
 
-- `public`: optional, if present, the ADT and its variants will be exported.
 - `<name>`: the name of the ADT, can be just a name or name with supertype.
 - `<body>`: the body of the ADT, a list of variants in `begin ... end` block.
 
@@ -242,7 +206,9 @@ function emit(def::ADTTypeDef, info::EmitInfo=EmitInfo(def))
         $(emit_exports(def, info))
         $(emit_struct(def, info))
         $(emit_variant_cons(def, info))
+        $(emit_variant_getproperty(def, info))
         $(emit_variant_binding(def, info))
+        $(emit_variant_type_show(def, info))
         $(emit_reflection(def, info))
         $(emit_getproperty(def, info))
         $(emit_propertynames(def, info))
@@ -257,19 +223,71 @@ xvariant_type(info::EmitInfo, idx::Int) = :(Core.bitcast($(info.typename), $(UIn
 xvariant_type(x) = xcall(Core, :getfield, x, QuoteNode(Symbol("#type")))
 
 function emit_variant_binding(def::ADTTypeDef, info::EmitInfo)
-    type_expr(idx) = xvariant_type(info, idx)
-    type_defs = expr_map(enumerate(def.variants)) do (idx, variant)
-        if variant.type === :singleton # instance is type
-            :(const $(variant.name) = $(def.name)($(type_expr(idx))))
+    def.export_variants || return :($nothing)
+
+    expr_map(enumerate(def.variants)) do (idx, variant)
+        type_enum = xvariant_type(info, idx)
+        if variant.type === :singleton
+            return :(const $(variant.name) = $(def.name)($type_enum))
         else
-            :(const $(variant.name) = $(type_expr(idx)))
+            return :(const $(variant.name) = $(type_enum))
+        end
+    end
+end
+
+function emit_variant_getproperty(def::ADTTypeDef, info::EmitInfo)
+    type_expr(idx) = xvariant_type(info, idx)
+
+    body = JLIfElse()
+    for (idx, variant) in enumerate(def.variants)
+        body[:(name === $(QuoteNode(variant.name)))] = if variant.type === :singleton
+            quote # instance is type
+                $(def.name)($(type_expr(idx)))
+            end
+        else
+            quote
+                $(type_expr(idx))
+            end
         end
     end
 
+    builtin_names = (
+        fieldnames(DataType)...,
+        fieldnames(Union)...,
+        fieldnames(UnionAll)...,
+    )
+    @static if VERSION < v"1.8-"
+        body[:(name in $builtin_names)] = :($Base.getfield(Self, name))
+    else
+        body[:(name in $builtin_names)] = :(@inline $Base.getfield(Self, name))
+    end
+    body.otherwise = :(throw(ArgumentError("invalid variant type")))
+
+    variant_names = map(def.variants) do variant
+        QuoteNode(variant.name)
+    end
+
+    return quote
+        function $Base.getproperty(::Type{Self}, name::Symbol) where {Self <:$(def.name)}
+            return $(codegen_ast(body))
+        end
+
+        function $Base.propertynames(::Type{Self}) where {Self <:$(def.name)}
+            return $(xtuple(variant_names...))
+        end
+
+        function $Base.propertynames(::Type{Self}, private::Bool) where {Self <:$(def.name)}
+            private || return $Base.propertynames(Self)
+            return $(xtuple(variant_names..., builtin_names...))
+        end
+    end
+end
+
+function emit_variant_type_show(def::ADTTypeDef, info::EmitInfo)
     show_body = JLIfElse()
     for (idx, variant) in enumerate(def.variants)
-        show_body[:(t == $(type_expr(idx)))] = quote
-            print(io, $(string(def.name)), "::", $(string(variant.name)))
+        show_body[:(t == $(xvariant_type(info, idx)))] = quote
+            print(io, $(string(def.name)), ".", $(string(variant.name)))
         end
     end
 
@@ -278,9 +296,7 @@ function emit_variant_binding(def::ADTTypeDef, info::EmitInfo)
     end
 
     return quote
-        $(type_defs)
-
-        function Base.show(io::IO, t::$(info.typename))
+        function $Base.show(io::IO, t::$(info.typename))
             $(codegen_ast(show_body))
             return
         end
@@ -309,31 +325,10 @@ function struct_cons(def::ADTTypeDef, info::EmitInfo)
 
             argname = args[idx]
             arg_idx = findfirst(isequal(idx), typeinfo.mask)
-            vtypenames = typeinfo.variant_types[arg_idx]
             type_guess = typeinfo.guess[arg_idx]
             jl = JLIfElse()
-            if !isempty(vtypenames)
-                @gensym variant_type
-                msg = "expect $(join(vtypenames, " or "))"
-                vtypes = map(vtypenames) do t
-                    :(Core.bitcast($(info.typename), $(info.name_type_map[t])))
-                end
-                jl[:(args[$arg_idx] isa $(def.name))] = quote
-                    $argname = args[$arg_idx]
-                    $variant_type = $ADT.variant_type($argname)
-                    $variant_type in $(xtuple(vtypes...)) || throw(ArgumentError(
-                        "$($msg), got $($variant_type)"))
-                end
-
-                if type_guess !== def.name
-                    jl[:(args[$arg_idx] isa $type_guess)] = quote
-                        $argname = args[$arg_idx]
-                    end
-                end
-            else
-                jl[:(args[$arg_idx] isa $type_guess)] = quote
-                    $argname = args[$arg_idx]
-                end
+            jl[:(args[$arg_idx] isa $type_guess)] = quote
+                $argname = args[$arg_idx]
             end
 
             jl.otherwise = quote
@@ -365,6 +360,8 @@ function struct_cons(def::ADTTypeDef, info::EmitInfo)
 end
 
 function emit_exports(def::ADTTypeDef, info::EmitInfo)
+    def.export_variants || return nothing
+
     names = map(def.variants) do variant
         return variant.name
     end
@@ -375,6 +372,7 @@ end
 function emit_struct(def::ADTTypeDef, info::EmitInfo)
     def = JLStruct(;
         def.name, def.typevars, info.ismutable,
+        def.supertype,
         fields=map(info.fieldnames, info.fieldtypes) do name, type
             JLField(;name, type)
         end,
@@ -394,7 +392,7 @@ function emit_variant_cons(def::ADTTypeDef, info::EmitInfo)
         nfields = length(variant.fieldtypes)
         assign_kwargs = expr_map(1:nfields,
             variant.fieldnames, variant.field_defaults) do idx, name, default
-            
+
             var = Symbol("#kw#", idx)
             msg = "missing keyword argument: $(name)"
             throw_ex = Expr(:block, variant.lineinfo, :(throw(ArgumentError($msg))))
