@@ -1,8 +1,16 @@
+module TestEmit
+
 using Test
 using Expronicon
-using Expronicon.ADT: ADT, EmitInfo, ADTTypeDef, @adt, emit_struct, emit_exports,
-    emit_show, emit_variant_cons, emit_reflection, emit_variant_binding,
-    emit_getproperty, emit_propertynames,
+using Expronicon.ADT: ADT, EmitInfo, ADTTypeDef, @adt, @use, @const_use,
+    emit_exports,
+    emit_struct,
+    emit_show,
+    emit_variant_cons,
+    emit_reflection,
+    emit_variant_binding,
+    emit_getproperty,
+    emit_propertynames,
     # reflection
     emit_variants,
     emit_variant_type,
@@ -40,6 +48,8 @@ body = quote
     ChangeColor(::Int64, ::Int64, ::Int64)
 end
 
+def = ADTTypeDef(Main, :Message, body)
+info = EmitInfo(def)
 
 @testset "EmitInfo(::ADTTypeDef)" begin
     def = ADTTypeDef(Main, :Message, body)
@@ -83,7 +93,6 @@ end
 
 def = ADTTypeDef(Main, :Message, body)
 info = EmitInfo(def)
-@test_expr emit_exports(def, info) == :(export Quit, Move, Write, Aka, ChangeColor, Message)
 
 @test_expr emit_struct(def, info) == quote
     #= /Users/roger/Code/Julia/Expronicon/src/adt/emit.jl:329 =# Core.@__doc__ struct Message
@@ -277,7 +286,7 @@ end
 
 @test_expr emit_variants(def, info) == quote
     function (Expronicon.ADT).variants(::Type{<:Message})
-        return (Quit, Move, Write, Aka, ChangeColor)
+        return (Core.bitcast(var"Message#Type", 0x00000001), Core.bitcast(var"Message#Type", 0x00000002), Core.bitcast(var"Message#Type", 0x00000003), Core.bitcast(var"Message#Type", 0x00000004), Core.bitcast(var"Message#Type", 0x00000005))
     end
 end
 
@@ -469,32 +478,21 @@ end
     end
 end
 
-@test_expr emit_variant_binding(def, info) == quote
+@test_expr emit_variant_binding(def, info) == nothing
+@test_expr emit_exports(def, info) == nothing
+
+pub_def = ADTTypeDef(Main, :Message, body; export_variants=true)
+@test_expr emit_variant_binding(pub_def, info) == quote
     const Quit = Message(Core.bitcast(var"Message#Type", 0x00000001))
     const Move = Core.bitcast(var"Message#Type", 0x00000002)
     const Write = Core.bitcast(var"Message#Type", 0x00000003)
     const Aka = Core.bitcast(var"Message#Type", 0x00000004)
     const ChangeColor = Core.bitcast(var"Message#Type", 0x00000005)
-    function Base.show(io::IO, t::var"Message#Type")
-        if t == Core.bitcast(var"Message#Type", 0x00000001)
-            print(io, "Message", "::", "Quit")
-        elseif t == Core.bitcast(var"Message#Type", 0x00000002)
-            print(io, "Message", "::", "Move")
-        elseif t == Core.bitcast(var"Message#Type", 0x00000003)
-            print(io, "Message", "::", "Write")
-        elseif t == Core.bitcast(var"Message#Type", 0x00000004)
-            print(io, "Message", "::", "Aka")
-        elseif t == Core.bitcast(var"Message#Type", 0x00000005)
-            print(io, "Message", "::", "ChangeColor")
-        else
-            throw(ArgumentError("invalid variant type"))
-        end
-        return
-    end
 end
 
+@test_expr emit_exports(pub_def, info) == :(export Quit, Move, Write, Aka, ChangeColor, Message)
 
-@adt MubanLang begin
+body = quote
     None
     # reference to a Julia variable
     Id(::Symbol)
@@ -510,12 +508,8 @@ end
         stmts::Vector{MubanLang}
     end
 end
-
-@testset "variant as field type" begin
-    @test Reference(Id(:x), Id(:y), None).some == None
-    @test Template([Id(:x), Id(:y), None]) isa MubanLang
-    @test_throws ArgumentError Reference(None, Id(:y), None)
-end
+def = ADTTypeDef(Main, :MubanLang, body)
+@test_throws ErrorException EmitInfo(def)
 
 @testset "Vector{Variant}" begin
     body = quote
@@ -578,3 +572,5 @@ info = EmitInfo(def)
             end
         end
 end
+
+end # module
