@@ -2,7 +2,7 @@ module TestEnum
 
 using Test
 using MLStyle
-using Expronicon.ADT: ADT, @adt, ADTTypeDef, EmitInfo,
+using Expronicon.ADT: ADT, @adt, @use, ADTTypeDef, EmitInfo,
     emit_variant_cons, variant_fieldnames, variant_masks,
     variant_type, variants, variant_typename, adt_type
 
@@ -14,6 +14,8 @@ using Expronicon.ADT: ADT, @adt, ADTTypeDef, EmitInfo,
     ChannelTypeTooLong(::Int)
 end
 
+@use AddressMaskErr: *
+
 @testset "singleton enum match" begin
     e = BinLengthNotMatch(1, 2)
     @match e begin
