@@ -1,9 +1,9 @@
 module TestExport
 
 using Test
-using Expronicon.ADT: @adt
+using Expronicon.ADT: @adt, @export_use
 
-@adt public PulseLang begin
+@adt PulseLang begin
     struct Waveform
         coeff::Vector{Float64}
         mask::BitVector
@@ -12,9 +12,20 @@ using Expronicon.ADT: @adt
     end
 end
 
+export PulseLang
+@export_use PulseLang: *
+
+@adt public Foo begin
+    A(::Int)
+    B(::Float64)
+end
+
 @testset "TestExport" begin
     @test :PulseLang in names(TestExport)
     @test :Waveform in names(TestExport)
+    @test :Foo in names(TestExport)
+    @test :A in names(TestExport)
+    @test :B in names(TestExport)
 end
 
 end # TestExport
