@@ -2,7 +2,7 @@ module TestIsEqualEnum
 
 using Test
 using MLStyle
-using Expronicon.ADT: @adt, variant_type
+using Expronicon.ADT: @adt, @use, variant_type
 
 @adt Foo begin
     Bar
@@ -11,6 +11,8 @@ using Expronicon.ADT: @adt, variant_type
     end
 end
 
+@use Foo: *
+
 function Base.:(==)(lhs::Foo, rhs::Foo)
     @match (lhs, rhs) begin
         (Bar, Bar) => true
